# Cranc - A pagure CLI
# Copyright (C) 2019 Lenka Segura
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
import json
import logging
import os

import click
import pprint

from libpagure import libpagure
from libpagure import exceptions

api_token = os.getenv("CRANC_TOKEN")
project = "pagure"

_log = logging.getLogger(__name__)

# Creating a Pagure object
PAGURE = libpagure.Pagure(pagure_token=api_token, pagure_repository=project)

# Creating group cranc
@click.group()
def cranc():
    pass

# Creating subgroup 'get'
@click.group()
def get():
    pass


@click.command(name="prs")
@click.option("--status")
@click.option("--assignee")
@click.option("--author")
def pr_list(status, assignee, author):
    """Prints list of pull requests"""
    prs = filter_pull_requests(
        status=status, assignee=assignee, author=author
    )
    pprint.pprint(prs)


def filter_pull_requests(status, assignee, author):
    """ 
    Get all pull requests of a project.
    :param status: filters the status of the requests
    :param assignee: filters the assignee of the requests
    :param author: filters the author of the requests
    :return:
    """
    try:
        return PAGURE.list_requests(status=status, assignee=assignee, author=author)
    except Exception:
        _log.exception("Failed to connect to the server")


# Adding pr_list into the group 'get'
get.add_command(pr_list)

@click.command(name="issues")
@click.option("--status")
@click.option("--tags")
@click.option("--assignee")
@click.option("--author")
@click.option("--milestones")
@click.option("--priority")
@click.option("--no_stones")
@click.option("--since")
#@click.option("--order")
def issue_list(status, tags, assignee, author, milestones, priority, no_stones,
        since):
    """Prints list of issues"""
    try:
        issues = PAGURE.list_issues(status=status, tags=tags, assignee=assignee,
                author=author, milestones=milestones, priority=priority,
                no_stones=no_stones, since=since)
    except exceptions.APIError:
        pagure_noauth = libpagure.Pagure(pagure_repository=project)
        issues = pagure_noauth.list_issues(status=status, tags=tags, assignee=assignee,
                author=author, milestones=milestones, priority=priority,
                no_stones=no_stones, since=since)
    pprint.pprint(issues)


# Adding issue_list into the group 'get'
get.add_command(issue_list)

# Create subgroup 'merge'
@click.group()
def merge():
    pass

@click.command(name="pr")
@click.option("--request_id")
def merge_pr(request_id):
    """This command merges a pull request"""
    try:
        request = PAGURE.merge_request(request_id=request_id)
        pprint.pprint(request)
        print("Pull request succesfully merged")
    except Exception:
        _log.exception("Failed to connect to the server")

# Adding merge_pr into the group 'merge'
merge.add_command(merge_pr)

# Create subgroup 'create'
@click.group()
def create():
    pass

@click.command(name="pr")
@click.option("--repo")
@click.option("--title")
@click.option("--branch_to")
@click.option("--branch_from")
def create_pr(repo, title, branch_to, branch_from):
    """this command creates a new pull request"""
    try:
        request = PAGURE.create_pull_request(repo=repo, title=title, branch_to=branch_to,
               branch_from=branch_from)
        pprint.pprint(request)
    except Exception:
        _log.exception("Failed to connect to the server")

# Adding create_pr into the group 'create'
create.add_command(create_pr)

################################################################################
# General section
################################################################################

cranc.add_command(get)
cranc.add_command(merge)
cranc.add_command(create)
